@@ -30,25 +30,33 @@ LOG = logging.getLogger(__name__)
 POC_PARAMS = {'controller': 1, 'compute': 2}
 
 
-def parse_counts(counts):
+def parse_counts_and_flavors(counts, overcloud_roles):
     """Helper for parsing the OvercloudRoleCount object
 
-    Given a list of OvercloudRoleCount objects return a dict of
-    (image_name, count) in a format used for building a template.
+    Given a list of OvercloudRoleCount and dict of OverlcoudRole objects
+    return a dict of (image_name, count) and (image_name, flavor_id) in a
+    format used for building a template.
 
     :param counts: List of tuskar.api.controllers.v1.models.OvercloudRoleCount
     :type  counts: list
 
-    :return: Dict of (image_name, count)
-    :rtype:  dict
+    :param overcloud_roles: Dict of (overcloud_role_id, overcloud_role) so
+                            we can access image_name and flavor_id of roles
+    :type  overcloud_roles: dict
+
+    :return: Tuple of dicts {(image_name, count)}, {(image_name, flavor_id)}
+    :rtype:  two dict objects
     """
     parsed_counts = {}
+    parsed_flavors = {}
     for count_obj in counts:
-        image_name = count_obj.overcloud_role.image_name
+        image_name = overcloud_roles[count_obj.overcloud_role_id].image_name
+        flavor_id = overcloud_roles[count_obj.overcloud_role_id].flavor_id
         count = count_obj.num_nodes
         parsed_counts[image_name] = count
+        parsed_flavors[image_name] = flavor_id
 
-    return parsed_counts
+    return parsed_counts, parsed_flavors
 
 
 def filter_template_attributes(allowed_data, attributes):
@@ -76,7 +84,37 @@ def filter_template_attributes(allowed_data, attributes):
     return filtered_data
 
 
-def process_stack(attributes, counts, create=False):
+def get_overcloud_roles_dict():
+    return dict((overcloud_role.id, overcloud_role)
+                for overcloud_role in
+                pecan.request.dbapi.get_overcloud_roles())
+
+
+def get_flavor_attributes(parsed_flavors):
+    """Helper for building dict of flavor attributes
+
+    Given a dict of parsed flavors, it will put a flavor_ids stored in
+    role into attributes that will be fed to heat stack create/update.
+
+    Mapping of image name to flavor_param is stored in template_tools.ROLES.
+
+    :param parsed_flavors: Dict of (image_name, flavor_id)
+    :type  parsed_flavors: dict
+
+    :return: Dict of (flavor_param, flavor_id) for Heat Template params
+    :rtype:  dict
+    """
+    flavor_attributes = {}
+    for image_name, flavor_id in parsed_flavors.items():
+        role = template_tools.ROLES.get(image_name, None)
+        if role:
+            flavor_param = role['flavor_param']
+            flavor_attributes[flavor_param] = flavor_id
+
+    return flavor_attributes
+
+
+def process_stack(attributes, counts, overcloud_roles, create=False):
     """Helper function for processing the stack.
 
     Given a params dict containing the Overcloud Roles and initialization
@@ -89,42 +127,66 @@ def process_stack(attributes, counts, create=False):
     :param counts: Dictionary of counts of roles to be deployed
     :type  counts: dict
 
+    :param overcloud_roles: Dict of (overcloud_role_id, overcloud_role) so
+                            we can access image_name and flavor_id of roles
+    :type  overcloud_roles: dict
+
     :param create: A flag to designate if we are creating or updating the stack
     :type create: bool
     """
+    heat_client = HeatClient()
+
     try:
-        overcloud = template_tools.merge_templates(parse_counts(counts))
+        # Get how many of each role we want and what flavor each role uses.
+        parsed_counts, parsed_flavors = parse_counts_and_flavors(
+            counts, overcloud_roles)
+    except Exception as e:
+        raise exception.ParseCountsAndFlavorsFailed(six.text_type(e))
+
+    try:
+        # Build the template
+        overcloud = template_tools.merge_templates(parsed_counts)
     except Exception as e:
         raise exception.HeatTemplateCreateFailed(six.text_type(e))
 
-    heat_client = HeatClient()
-
-    stack_exists = heat_client.exists_stack()
     try:
+        # Get the parameters that the template accepts and validate
         allowed_data = heat_client.validate_template(overcloud)
     except Exception as e:
         raise exception.HeatTemplateValidateFailed(six.text_type(e))
 
+    stack_exists = heat_client.exists_stack()
     if stack_exists and create:
         raise exception.StackAlreadyCreated()
 
     elif not stack_exists and not create:
         raise exception.StackNotFound()
 
+    try:
+        # Put flavors from OverloudRoles into attributes
+        attributes.update(get_flavor_attributes(parsed_flavors))
+
+        # Filter the attributes to allowed only
+        filtered_attributes = filter_template_attributes(allowed_data,
+                                                         attributes)
+    except Exception as e:
+        raise exception.HeatStackProcessingAttributesFailed(six.text_type(e))
+
     if create:
         operation = heat_client.create_stack
     else:
         operation = heat_client.update_stack
 
     try:
-        operation(overcloud,
-                  filter_template_attributes(allowed_data, attributes))
+        result = operation(overcloud, filtered_attributes)
     except Exception as e:
         if create:
             raise exception.HeatStackCreateFailed(six.text_type(e))
         else:
             raise exception.HeatStackUpdateFailed(six.text_type(e))
 
+    return result
+
 
 class OvercloudsController(rest.RestController):
     """REST controller for the Overcloud class."""
@@ -156,7 +218,19 @@ class OvercloudsController(rest.RestController):
         """
         LOG.debug('Creating overcloud: %s' % transfer_overcloud)
 
+        # FIXME(lsmola) This is just POC of creating a stack
+        # this has to be done properly with proper Work-flow abstraction of:
+        # step 1- build template and start stack-create
+        # step 2- put the right stack_id to the overcloud
+        # step 3- initialize the stack
+        # step 4- set the correct overcloud status
+        stack = process_stack(transfer_overcloud.attributes,
+                              transfer_overcloud.counts,
+                              get_overcloud_roles_dict(),
+                              create=True)
+
         # Persist to the database
+        transfer_overcloud.stack_id = stack['stack']['id']
         db_overcloud = transfer_overcloud.to_db_model()
         result = pecan.request.dbapi.create_overcloud(db_overcloud)
 
@@ -164,15 +238,6 @@ class OvercloudsController(rest.RestController):
         saved_overcloud =\
             models.Overcloud.from_db_model(result)
 
-        # FIXME(lsmola) This is just POC of creating a stack
-        # this has to be done properly with proper Work-flow abstraction of:
-        # step one- build template and start stack-create
-        # step 2- put the right stack_id to the overcloud
-        # step 3- initialize the stack
-        # step 4- set the correct overcloud status
-        process_stack(saved_overcloud.attributes, result.counts,
-                      create=True)
-
         return saved_overcloud
 
     @wsme.validate(models.Overcloud)
@@ -210,10 +275,10 @@ class OvercloudsController(rest.RestController):
 
         # FIXME(lsmola) This is just POC of updating a stack
         # this probably should also have workflow
-        # step one- build template and stack-update
+        # step 1- build template and stack-update
         # step 2- set the correct overcloud status
-        process_stack(updated_overcloud.attributes, updated_overcloud.counts,
-                      create=True)
+        process_stack(updated_overcloud.attributes, result.counts,
+                      get_overcloud_roles_dict())
 
         return updated_overcloud
 
@@ -231,7 +296,7 @@ class OvercloudsController(rest.RestController):
         # FIXME(lsmola) this should always try to delete both overcloud
         # and stack. So it requires some exception catch over below.
         # FIXME(lsmola) there is also a workflow needed
-        # step one- delete stack and set status deleting in progress to
+        # step 1- delete stack and set status deleting in progress to
         # overcloud
         # step 2 - once stack is deleted, delete the overcloud
         LOG.debug('Deleting overcloud with ID: %s' % overcloud_id)
